#include "Cgiu.h"

// device and connection constants:

#ifdef linux 
#define SERIALDEVICE1 "/dev/ttyS0" // linux
#define SERIALDEVICE2 "/dev/ttyS1"
#else 
#define SERIALDEVICE1 "/dev/ttya"  // solaris
#define SERIALDEVICE2 "/dev/ttyb" 
#endif



#define BAUDRATE B38400 // default baudrate
#define PARITY CS8 // default parity = no parity

// ***** private funtions: *****

// SetTimeout(timeout): setting the VTIME flag (time measured in steps of 1/10 sec)
void Cgiu::SetTimeout(int fd) {
  parameters.c_cc[VTIME] = 0;
  parameters.c_cc[VMIN] = 0;
  tcsetattr(fd, TCSANOW, &parameters);
}

// SetTransferProperties(baudrate, parity) is for setting baudrate and parity manually
void Cgiu::SetTransferProperties(int fd, int baudrate, int parity) {
  parameters.c_cflag = baudrate | parity | CREAD | CLOCAL;
  parameters.c_oflag = 0;
  parameters.c_lflag = 0;
  parameters.c_iflag = 0; // no IGNPAR 
  tcsetattr(fd, TCSANOW, &parameters);
}

// ReadData(num, timeout): num bytes are read separately. Returns 1 on success or -1 on (timeout) error
int Cgiu::ReadData(int fd, int num) {
  int check = read(fd, readBuffer, num);
  return check;
}

// ***** public functions: *****

// constructor
Cgiu::Cgiu() {
  readBuffer = new unsigned char [128];
}

// destructor
Cgiu::~Cgiu() {
  delete(readBuffer);
}

// Init() opens the device defined by SERIALDEVICE and initializes
// with default values. Returns -1 on error!
int Cgiu::Init(char* device) {
  int tmp = open(device, O_RDWR);
  if (tmp == -1) {
  	printf("Alert: Error opening serial device\n");
  	return -1;
  }
  tcgetattr(tmp, &parameters); // get device parameters and store in global variable parameters
  SetTransferProperties(tmp, BAUDRATE, PARITY);
  SetTimeout(tmp); // Vmin = 0; VTime = 0;
  return tmp;
}
