#include <curses.h>
#include "Cgiu.h"
#include "keypressed.h"
#include <stdio.h>
#include <stdlib.h>

#ifdef linux 
#define SERIALDEVICE1 "/dev/ttyS0" // linux
#define SERIALDEVICE2 "/dev/ttyS1"
#else 
#define SERIALDEVICE1 "/dev/ttya"  // solaris
#define SERIALDEVICE2 "/dev/ttyb" 
#endif

void pos(int x,int y) {
   printf("\033[%i;%if",y,x);
}

void cls() {
   printf("\033[2J");
}

void viewconsole() {
   cls();
   pos(1,1);
   printf("                       looping through the serial ports\n\n");
   printf("/-----------------------------------------------------------------------------\\\n");
   printf("| serial1 input                        | serial2 input                        |\n");
   printf("|-----------------------------------------------------------------------------|\n");
   for (int i = 0; i < 4; i++) {
      printf("|                                      |                                      |\n");
   }
   printf("|-----------------------------------------------------------------------------|\n");
   printf("| serial2 output                       | serial1 output                       |\n");
   printf("|-----------------------------------------------------------------------------|\n");
   for (int i = 0; i < 4; i++) {
      printf("|                                      |                                      |\n");
   }
   printf("\\-----------------------------------------------------------------------------/\n\n\n");  	
   printf(" 'e': delete one buffer packet           't': delete one buffer packet\n\n\n");
   printf("                                                              'q': quit program\n");
}

void charout(unsigned char c) {
   if ((c > 30) && (c < 128)) printf("%c", c); else printf(".");
}

void addtobuf(unsigned char* src, unsigned char* dest, int len) {
   for (int i = (4*36-1); i >= len; i--) {
      dest[i] = dest[i-len];
   }
   for (int i = 0; i < len; i++) {
      dest[i] = src[i];
   }
}

void flushbuf(unsigned char* buf) {
 for (int i = 0; i < 255; i++) buf[i] = ' '; 
}

void main(int argc, char *argv[]) {
   beep;
   flash;
   
   
   Cgiu myCgiu;

   int fd1 = myCgiu.Init(SERIALDEVICE1);
   if (fd1 != -1) printf("oeffnen von serial1 erfolgreich\n"); else exit(-1);
   int fd2 = myCgiu.Init(SERIALDEVICE2);
   if (fd2 != -1) printf("oeffnen von serial2 erfolgreich\n"); else exit(-1);

   // Konsolen-Gerst anzeigen
   
   viewconsole();	 
	 	   	
   sleep(1);
   
   //exit(-1);
   
   int check;
   int errorrate1 = 150;
   int errorrate2 = 150;
   int noerrors1 = 0;
   int noerrors2 = 0;
   int loopthrough1 = 1;
   int loopthrough2 = 1;
   int deletepacket1 = 0;
   int deletepacket2 = 0;
   char cc = ' ';
   unsigned char buf1[255]; flushbuf(buf1);
   unsigned char buf2[255]; flushbuf(buf2);
   unsigned char buf3[255]; flushbuf(buf3);
   unsigned char buf4[255]; flushbuf(buf4); 
   
   tty_noncanonical(STDIN_FILENO); // f. Tastatureingaben
   
   // HAUPTSCHLEIFE
   
   do {
      // Puffer ausgeben...
      for (int i1 = 0; i1 < 4; i1++) {
         pos(3,6+i1);
         for (int i2 = i1*36; i2 < (i1+1)*36; i2++) charout(buf1[i2]);
         pos(3,13+i1);
         for (int i2 = (i1*36); i2 < (i1+1)*36; i2++) charout(buf2[i2]);
         pos(42,6+i1);
         for (int i2 = (i1*36); i2 < (i1+1)*36; i2++) charout (buf3[i2]);
         pos(42,13+i1);
         for (int i2 = (i1*36); i2 < (i1+1)*36; i2++) charout (buf4[i2]);
      }
      
      pos(2,19);
      if (noerrors1 == 0) printf("error rate: %d ('w': inc, 's': dec)", errorrate1);
      else printf("no looping through errors!           \n");
      
      pos(42,19);
      if (noerrors2 == 0) printf("error rate: %d ('r': inc, 'f': dec)\n", errorrate2);
      else printf("no looping through errors!           \n");
      pos(2,21); 
      if (loopthrough1 == 1) printf("'d': stop loop through   "); else printf("'d': start loop through   ");
      pos(42,21);
      if (loopthrough2 == 1) printf("'g': stop loop through   "); else printf("'g': start loop through   ");
      
      // usleep(10); // a bissal schlafen...
      
      pos(1,23);
     
      if (keypressed(cc) == 1) {      
         switch (cc) {
            case 'w': {
               errorrate1++;
               noerrors1 = 0;
               printf("error rate (serial1->serial2) increased           \n");
               break;
            }
            case 'W': {
               printf("no errors for serial1->serial2                      \n");
               noerrors1 = 1;
               break;
            }
            case 's': {
               if (errorrate1 > 1) {
                  errorrate1--;
                  noerrors1 = 0;
                  printf("error rate (serial1->serial2) decreased           \n");
               } else printf("                                                  \n");
               break;
            }
            case 'r': {
               errorrate2++;
               noerrors2 = 0;
               printf("error rate (serial2->serial1) increased           \n");
               break;
            }
            case 'R': {
               printf("no errors for serial2->serial1                      \n");
               noerrors2 = 1;
               break;
            }
            case 'f': {
               if (errorrate2 > 1) {
                  errorrate2--;
                  noerrors2 = 0;
                  printf("error rate (serial2->serial1) decreased           \n");
               } else printf("                                                  \n");
               break;
            }
            case 'e': {
               printf("killing one buffer packet (serial1->serial2)       \n");
               deletepacket1 = 1;
               break;
            }
            case 't': {
               printf("killing one buffer packet (serial2->serial1)       \n");
               deletepacket2 = 1;
               break;
            }
            case 'd': {
               loopthrough1 = (1 - loopthrough1);
               if (loopthrough1 == 1) printf("data flow started (serial1->serial2)              \n");
                 else printf("data flow stopped (serial1->serial2)            \n"); 
               break;
            }
            case 'g': {
               loopthrough2 = (1 - loopthrough2);
               if (loopthrough2 == 1) printf("data flow started (serial2->serial1)              \n");
                 else printf("data flow stopped (serial2->serial1)            \n"); 
               break;
            }
            case ('c'): {
               flushbuf(buf1);
               flushbuf(buf2);
               flushbuf(buf3);
               flushbuf(buf4);
               printf("Clear Screen                       \n"); 
               break;
            }
            
            default: 
               printf("                                                  \n");
               
         }
      }
      
      check = myCgiu.ReadData(fd1, 255);
      if (check != 0) {  
         addtobuf(myCgiu.readBuffer, buf1, check);
         if ((((int)(rand()%errorrate1) != 1) || (noerrors1 == 1))
             && (loopthrough1 == 1) && (deletepacket1 == 0)) {
            write(fd2, myCgiu.readBuffer, check);
            addtobuf(myCgiu.readBuffer, buf2, check);
         }
         deletepacket1 = 0;
      }
      
      check = myCgiu.ReadData(fd2, 255);
      if (check != 0) {  
         addtobuf(myCgiu.readBuffer, buf3, check);
         if ((((int)(rand()%errorrate2) != 1) || (noerrors2 == 1))
             && (loopthrough2 == 1) && (deletepacket2 == 0)) {
            write(fd1, myCgiu.readBuffer, check);
            addtobuf(myCgiu.readBuffer, buf4, check);
         }
         deletepacket2 = 0;
      }
     
   } while (cc != 'q');
   tty_reset(STDIN_FILENO);    
   pos(1,24);
   printf("\n");
   
}
